import sys

from ntsim.Primary import *
from ntsim.primaryPropagators.g4LiGen import g4LiGen
from ntsim.primaryPropagators.g4CaGen import g4CaGen
from ntsim.primaryPropagators.cascadeCherenkov import cascadeCherenkov
from ntsim.primaryPropagators.trackCherenkov import trackCherenkov
from ntsim.utils.report_timing import report_timing

class G4Particle(Primary):
    def __init__(self):
        super().__init__()
        self.vertices = []
        self.tracks = []
        self.hits = []
        self.log = logging.getLogger('G4ParticlePrimary')
        self.primary_propagator = {}

    def configure(self, opts):
        # it will generate primaries and propagate them in the same module
        self.configure_propagators(opts)
        self.photon_suppression = opts.photon_suppression
        self.log.info(f"primary propagators: {[key for key in self.primary_propagator.keys()]}")
        self.photon_suppression = opts.photon_suppression
        self.log.info('configured')

    @report_timing
    def next(self, thread_id=0):
        photons = Photons()
        self.make_event_folder(self.gEvent.gProductionHeader.n_events) #FIXME : should not it be in the parent class?
        if 'g4LiGen' in self.primary_propagator.keys():
            self.vertices, self.tracks, photons = self.primary_propagator['g4LiGen'].propagate()
            self.log.info(f"{len(self.tracks)} tracks simulated")
            self.log.info(f"{photons.n_tracks} photons produced (1/{self.photon_suppression} of real number)")
        #
        if 'g4CaGen' in self.primary_propagator.keys():
            self.vertices, self.tracks = self.primary_propagator['g4CaGen'].propagate()

        #
        if 'cascadeCherenkov' in self.primary_propagator.keys():
            new_photons = self.primary_propagator['cascadeCherenkov'].propagate(self.vertices)
            self.log.info(f"{new_photons.n_tracks} photons in cascades (1/{self.photon_suppression} of real number)")
            photons = photons.add_photons(new_photons)
        #
        if 'trackCherenkov' in self.primary_propagator.keys():
            new_photons = self.primary_propagator['trackCherenkov'].propagate(self.tracks)
            self.log.info(f"{new_photons.n_tracks} photons from tracks (1/{self.photon_suppression} of real number)")
            photons = photons.add_photons(new_photons)
        #
        if 'mcPhotonTransporter' in self.primary_propagator.keys():
            self.clear_hits()
            self.hits = self.primary_propagator['mcPhotonTransporter'].transport(photons,
                                                                                 self.medium.get_model(photons),
                                                                                 self.geometry)
            #self.write_hits()
            self.log.info(f"{len(np.unique(self.hits[:,0]))} OMs fired")
            self.log.info(f"{self.hits.shape[0]} hits detected")
        #
        if self.tracks != []:
            self.write_tracks()
        if self.hits != []:
            self.write_hits()
        if photons != None:
            bunch_id = 0
            self.write_photons_bunch(photons, bunch_id)
            self.write_number_of_bunches(bunch_id+1, photons.n_tracks)
        #
        self.write_event_header()


    def next_multithread(self, n, jobs=0):
#        self.next()
        from multiprocessing.pool import ThreadPool
        from multiprocessing import cpu_count
        if jobs == 0:
            jobs = cpu_count()
        t = ThreadPool(processes=jobs)
        t.map(self.next, range(1,n))
        t.close()

    def get_photons_sampling_weight(self):
        return self.photon_suppression

    def get_vertices(self):  #FIXME: do we need vertices?
        return self.vertices
