import numpy as np
from g4camp.g4camp import g4camp
from ntsim.utils.report_timing import report_timing

import logging
log = logging.getLogger('g4CaGen')


class g4CaGen(g4camp):

    def __init__(self):
        super().__init__(optics=False, primary_generator='gun')

    def configure(self, opts):
        # pre-init actions
        self.skip_min = opts.g4cagen_casc_min
        self.skip_max = opts.g4cagen_casc_max
        self.setSkipMinMax(self.skip_min, self.skip_max)
        #
        super().configure() # /run/initialize happens here
        #
        # post-init actions
        self.setGunParticle(opts.g4_particle)
        self.setGunEnergy(float(opts.g4_energy[0]), opts.g4_energy[1])
        self.setGunPosition(float(opts.g4_position[0]), float(opts.g4_position[1]), 
                            float(opts.g4_position[2]), opts.g4_position[3])
        self.setGunDirection(float(opts.g4_direction[0]), float(opts.g4_direction[1]), 
                             float(opts.g4_direction[2]))
        log.info("configured")

    def print_vertex_info(self, vertices):
        log.info(f"number of vertices : {len(vertices)}")
        pdgids, counts = np.unique([vertex[2][0][0] for vertex in vertices], return_counts=True)
        for pdgid, count in zip(pdgids, counts):
            log.debug(f"      {pdgid:<12} : {count}")

    def print_track_info(self, tracks):
        n_steps = sum([len(tracks[key].points) for key in tracks.keys()])
        log.info(f"number of tracks and steps : {len(tracks)} / {n_steps}")
        pdgids, counts = np.unique([tracks[key].pdg_id for key in tracks.keys()], return_counts=True)
        for pdgid, count in zip(pdgids, counts):
            log.debug(f"      {pdgid:<10} : {count}")

    @report_timing
    def propagate(self):
        log.info("running")
        evt_data = next(self.run(1))
        vertices = evt_data.vertices
        tracks = evt_data.tracks
        self.print_vertex_info(vertices)
        self.print_track_info(tracks)
        return vertices, tracks
