import numpy as np
from g4camp.g4camp import g4camp
from ntsim.utils.report_timing import report_timing
from ntsim.photonTransporters.Photons import Photons

import logging
log = logging.getLogger('g4LiGen')


class g4LiGen(g4camp):

    def __init__(self):
        super().__init__(optics=True, primary_generator='gun')

    def configure(self, opts):
        # pre-init actions
        self.setSkipMinMax(0., 0.) # skip nothing
        self.setPhotonSuppressionFactor(opts.photon_suppression)
        #
        super().configure() # /run/initialize happens here
        #
        # post-init actions
        self.setGunParticle(opts.g4_particle)
        self.setGunEnergy(float(opts.g4_energy[0]), opts.g4_energy[1])
        self.setGunPosition(float(opts.g4_position[0]), float(opts.g4_position[1]), 
                            float(opts.g4_position[2]), opts.g4_position[3])
        self.setGunDirection(float(opts.g4_direction[0]), float(opts.g4_direction[1]), 
                             float(opts.g4_direction[2]))
        log.info("configured")

    def getPhotonFraction(self):
        return 1./self.ph_suppression_factor

    @report_timing
    def propagate(self):
        log.info("running")
        evt_data = next(self.run(1))
        vertices = evt_data.vertices
        tracks = evt_data.tracks
        photon_cloud = np.array(evt_data.photon_cloud)
        #
        n_photons = photon_cloud.shape[0]
        position = photon_cloud[:,0:3]
        time = photon_cloud[:,3]
        direction = photon_cloud[:,4:7]
        wavelength = photon_cloud[:,7]
        position = np.expand_dims(position, axis=0) #FIXME in Photon: only one step by default
        direction = np.expand_dims(direction, axis=0)
        time = np.expand_dims(time, axis=0)
        n_steps = 1
        photons = Photons()
        photons.init(n_photons, n_steps, position, time, direction, wavelength) # FIXME: why arguments are passed not in constructor as in chains?
        # FIXME: change name 'init' -> 'fill_data'? Otherwise missleading.
        return vertices, tracks, photons
